from loguru import logger
from upswingutil.db import Firestore, Mongodb
from upswingutil.resource import http_retry, FirebaseHelper
from upswingutil.schema import ResponseDict, CheckinReservationModel
from upswingutil.integrations.models import Providers


def doorlock_activation(orgId, data: CheckinReservationModel, auth_token: str) -> ResponseDict:
    """
    Generate doorlock for given reservation with respect to which doorlock provider is linked with the given room
    @author: harsh
    :param orgId:
    :param data: check-in details
    :param auth_token: for accessing upswing's messerchmitt server
    :return: response dict
    """
    response = ResponseDict()
    mongo = Mongodb(orgId)
    try:
        room_mapping = mongo.get_collection(mongo.IOT_ROOM_MAPPING).find_one(
            {'hotelId': data.hotelId, 'pmsRoomId': data.roomId})
        if room_mapping['doorlockProvider'] is not None and room_mapping['doorlockProvider'] != '' and room_mapping['doorlockId'] != '':
            if room_mapping['doorlockProvider'] == Providers.MESSERCHMITT.value:
                response = messerchmitt_doorlock_activation(orgId, data, room_mapping, auth_token)
            elif room_mapping['doorlockProvider'] == Providers.ASSA_ABLOY.value:
                raise Exception(f'Doorlock not implemented yet for {room_mapping["doorlockProvider"]}')
            else:
                raise Exception(f'Doorlock not implemented yet for {room_mapping["doorlockProvider"]}')

            if response.status:
                response = post_doorlock_creation(orgId, data, response.data)
        else:
            response.message = f'Doorlock not mapped for room {orgId} : {data.hotelId} : {data.roomId}'
            logger.info(response.message)
        response.status = True
    except Exception as e:
        response.message = f'Failed to generate doorlock for {orgId} : {data.reservationId}'
        logger.error(response.message)
        logger.error(e)
    finally:
        mongo.close_connection()
        return response


def post_doorlock_creation(orgId, data: CheckinReservationModel, doorLocks: dict) -> ResponseDict:
    """
    This function takes care of storing the generated doorlock to respective profiles and notifying the guest for the same
    @author: harsh
    :param orgId: the hotel chain
    :param data: check in details
    :param doorLocks: doorlock of each guest profile
    :return: response dict with details
    """
    response = ResponseDict()
    mongo = Mongodb(orgId)
    fireDB = Firestore('alvie')
    for uid in doorLocks.keys():
        data_to_update = {
            f'guest_app_data.{uid}.doorlockKey': doorLocks.get(uid).get('doorlockKey'),
            f'guest_app_data.{uid}.doorlockSpecialAccess': doorLocks.get(uid).get('doorlockSpecialAccess')
        }
        fireDB.get_collection(f'Organizations/{orgId}/reservations').document(data.reservationId).set({
            'guest_app_data': {
                uid: {
                    'doorlockKey': doorLocks.get(uid).get('doorlockKey'),
                    'doorlockSpecialAccess': doorLocks.get(uid).get('doorlockSpecialAccess')
                }
            }
        }, merge=True)
        mongo.get_collection(mongo.RESERVATION_COLLECTION).find_one_and_update(
            {'_id': data.reservationId},
            {
                '$set': data_to_update
            },
            upsert=True
        )
    mongo.close_connection()
    response.status = True
    response.message = f'Doorlock generated for {len(doorLocks.keys())} profiles in {orgId} : {data.reservationId}'
    logger.info(response.message)
    return response


def messerchmitt_doorlock_activation(orgId, data: CheckinReservationModel, room_mapping, auth_token) -> ResponseDict:
    """
    This function generated messerchmitt doorlock key for each guest in reservation
    @author: harsh
    :param orgId:
    :param data: check-in reservation details
    :param room_mapping: doorlock room to which this reservation is linked
    :param auth_token: to access upswing's messerchmitt server
    :return: response dict with status
    """
    response = ResponseDict()
    mongo = Mongodb(orgId)
    upswingDB = Mongodb('Upswing')
    fb = FirebaseHelper('alvie')
    header = {'accept': 'application/json', 'Content-Type': 'application/json', 'Authorization': 'Bearer ' + auth_token}
    http = http_retry()

    mstUpswingServer = upswingDB.get_collection("integrations").find_one(
        {
            'integration_type': 'doorlock',
            'provider': Providers.MESSERCHMITT.value,
            'solution_type': 'cloud'
        }
    )
    upswingDB.close_connection()
    _url = f"{mstUpswingServer['baseURL']}/v0/mst/doorlock/get/key"
    property_doorlock_config = mongo.get_collection(mongo.INTEGRATION_PROPERTY).find_one(
        {"provider": Providers.MESSERCHMITT.value, "hotelId": data.hotelId, "integration_type": "doorlock"},
        {"sa_guest": 1}
    )
    mongo.close_connection()
    special_area_request = ''
    if property_doorlock_config:
        special_area_request = property_doorlock_config['sa_guest']

    for guest in data.guest_list:
        guestUID = fb.find_user_by_email(guest.guestEmail)
        if guestUID:
            guestUID = guestUID.uid
            payload = {
                "orgId": orgId,
                "hotelId": data.hotelId,
                "doorlockRoomId": room_mapping['doorlockId'],
                "uid": guestUID,
                "sa_guest": special_area_request,
                "checkout_date": data.departureDate
            }
            try:
                result = http.post(url=_url, json=payload, headers=header, timeout=30)
                if result.status_code != 200:
                    response.message = f'Messerchmitt doorlock request failed {result.status_code}'
                    logger.error(response.message)
                    logger.error(f'URL: {result.url}')
                    logger.error(f'URL: {result.json()}')
                else:
                    result = result.json()
                    if result.get('status'):
                        response.data[guestUID] = {
                            'doorlockKey': result.get('data')['result'],
                            'doorlockSpecialAccess': special_area_request
                        }
                    else:
                        response.message = result.get('message')
                        logger.error(response.message)
            except Exception as e:
                logger.error(e)
    response.status = True
    response.message = 'Doorlock generated by Messerchmitt'
    return response



import json
import logging
import unittest

import firebase_admin
from firebase_admin import credentials

import upswingutil as ul
from upswingutil.db import Firestore
from upswingutil.grms import grms_activation
from upswingutil.resource import access_secret_version, setup_logging
from upswingutil.schema import CheckinReservationModel


class TestGRMS(unittest.TestCase):
    ul.G_CLOUD_PROJECT = 'aura-staging-31cae'
    ul.G_CLOUD_PROJECT_SECONDARY = 'alvie-development'
    ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
    ul.MONGO_URI = access_secret_version(ul.G_CLOUD_PROJECT, 'MONGOURI', '5')
    ul.LOG_LEVEL_VALUE = logging.DEBUG

    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Aura', '1'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred)

    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Alvie', '3'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred, name='alvie')

    setup_logging()

    def test_grms_activation(self):
        orgId = '11281'
        data = CheckinReservationModel(
            reservationId='169410',
            hotelId='11282',
            arrivalDate="2022-02-02 14:00:00",
            departureDate="2022-02-05 11:00:00",
            roomId="14",
            checkin_type="manual",
            doorlockLinked=True,
            grmsLinked=True,
            appLinked=True,
            guest_list=[
                {
                    "name": "Harsh Mathur",
                    "guestEmail": "harsh@upswing.global",
                    "primary": True,
                    "guestPMSId": "1234",
                    "appGuestUID": "nfploQfXyHdi14qvv8W1ZH62wkB2",
                    "appAccess": True,
                    "doorlockAccess": True,
                    "grmsAccess": True,
                    "guestType": "Primary"
                },
                {
                    "name": "Vikash Anand",
                    "guestEmail": "vikash@upswing.global",
                    "primary": True,
                    "guestPMSId": "1235",
                    "appGuestUID": "",
                    "appAccess": True,
                    "doorlockAccess": True,
                    "grmsAccess": False,
                    "guestType": "Co-traveller"
                }
            ]
        )
        token = 'eyJhbGciOiJSUzI1NiIsImtpZCI6IjUwYTdhYTlkNzg5MmI1MmE4YzgxMzkwMzIzYzVjMjJlMTkwMzI1ZDgiLCJ0eXAiOiJKV1QifQ.eyJvIjoiMTEyODEiLCJyIjoiQURNSU4iLCJpc3MiOiJodHRwczovL3NlY3VyZXRva2VuLmdvb2dsZS5jb20vYXVyYS1zdGFnaW5nLTMxY2FlIiwiYXVkIjoiYXVyYS1zdGFnaW5nLTMxY2FlIiwiYXV0aF90aW1lIjoxNjU3NTA4OTgwLCJ1c2VyX2lkIjoiWHV2SThEaFNjNGJlZ0pSclRBRjlFVWNYR2dNMiIsInN1YiI6Ilh1dkk4RGhTYzRiZWdKUnJUQUY5RVVjWEdnTTIiLCJpYXQiOjE2NTc1MTcxMTEsImV4cCI6MTY1NzUyMDcxMSwiZW1haWwiOiJkZW1vQHVwc3dpbmcuY2xvdWQiLCJlbWFpbF92ZXJpZmllZCI6ZmFsc2UsImZpcmViYXNlIjp7ImlkZW50aXRpZXMiOnsiZW1haWwiOlsiZGVtb0B1cHN3aW5nLmNsb3VkIl19LCJzaWduX2luX3Byb3ZpZGVyIjoicGFzc3dvcmQifX0.InMv4giaL8O8JHKqn0q223SeEKmNPGoc0kdNLjSvIVAsI5M2ATGxt4U9zmsxnJ_zOgJ1EkcT0S9BIPsiL4_49dnKR16bRXXl4SRu7LZnQb2E1BVFSVg5hUqUiEQaFfkX70Ac-IJpoGTjICkweLHF40jl_ZNVvdPdSB8akmqZ5cne661kv3BHa9-00lVR_9YVs2JbCQEI3zZwTSRhmE7SSQZNufUN9SfkxOhYq0QyP417_notazXBx6_IrDjV6lcNkZoeUrBd9XhJSsFWzQ3q5f1O4R0JPHMGZzO097Jp9GBFgZhEtO_gZGwdax664HcHA00xu3TogKECYU3CmcMxtw'
        response = grms_activation(orgId, data, token)
        assert response.status



import json
import logging
import unittest

import firebase_admin
from firebase_admin import credentials

import upswingutil as ul
from upswingutil.db import Firestore
from upswingutil.pms import checkin, post_check_in_reservation
from upswingutil.resource import access_secret_version, setup_logging
from upswingutil.schema import CheckinReservationModel


class TestCheckIn(unittest.TestCase):
    ul.G_CLOUD_PROJECT = 'aura-staging-31cae'
    ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
    ul.MONGO_URI = access_secret_version(ul.G_CLOUD_PROJECT, 'MONGOURI', '5')
    ul.LOG_LEVEL_VALUE = logging.DEBUG

    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Aura', '1'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred)

    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Alvie', '3'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred, name='alvie')

    setup_logging()

    def test_welcome_email_trigger(self):
        pass



import json
import logging
import unittest

import firebase_admin
from firebase_admin import credentials

import upswingutil as ul
from upswingutil.db import Firestore
from upswingutil.pms import checkin, post_check_in_reservation
from upswingutil.resource import access_secret_version, setup_logging
from upswingutil.schema import CheckinReservationModel


class TestCheckIn(unittest.TestCase):
    ul.G_CLOUD_PROJECT = 'aura-staging-31cae'
    ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
    ul.MONGO_URI = access_secret_version(ul.G_CLOUD_PROJECT, 'MONGOURI', '5')
    ul.LOG_LEVEL_VALUE = logging.DEBUG

    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Aura', '1'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred)

    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Alvie', '3'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred, name='alvie')

    setup_logging()

    def test_check_in_rms(self):
        orgId = '11281'
        data = CheckinReservationModel(
            reservationId='169410',
            hotelId='11282',
            arrivalDate="2022-02-02 14:00:00",
            departureDate="2022-02-05 11:00:00",
            roomId="14",
            checkin_type="manual",
            doorlockLinked=True,
            grmsLinked=True,
            appLinked=True,
            guest_list=[
                {
                    "name": "Harsh Mathur",
                    "guestEmail": "harsh@upswing.global",
                    "primary": True,
                    "guestPMSId": "1234",
                    "appGuestUID": "nfploQfXyHdi14qvv8W1ZH62wkB2",
                    "appAccess": True,
                    "doorlockAccess": True,
                    "grmsAccess": True,
                    "guestType": "Primary"
                },
                {
                    "name": "Vikash Anand",
                    "guestEmail": "vikash@upswing.global",
                    "primary": True,
                    "guestPMSId": "1235",
                    "appGuestUID": "",
                    "appAccess": True,
                    "doorlockAccess": True,
                    "grmsAccess": False,
                    "guestType": "Co-traveller"
                }
            ]
        )
        response = checkin(orgId, data)
        assert response.status
        assert response.message == 'Complete check-in done successfully'
        assert response.data['check_in_guest_count'] == len(data.guest_list)

    def test_check_in_opera(self):
        orgId = 'tajBusinessBay'
        data = CheckinReservationModel(
            hotelId='11282',
            reservationId='169410',
            arrivalDate="2022-02-02 14:00:00",
            departureDate="2022-02-05 11:00:00",
            roomId="14",
            checkin_type="manual",
            doorlockLinked=True,
            grmsLinked=True,
            appLinked=True,
            guest_list=[
                {
                    "name": "Harsh Mathur",
                    "guestEmail": "harsh@upswing.global",
                    "primary": True,
                    "guestPMSId": "1234",
                    "appGuestUID": "nfploQfXyHdi14qvv8W1ZH62wkB2",
                    "appAccess": True,
                    "doorlockAccess": True,
                    "grmsAccess": True,
                    "guestType": "Primary"
                },
                {
                    "name": "Vikash Anand",
                    "guestEmail": "vikash@upswing.global",
                    "primary": True,
                    "guestPMSId": "1235",
                    "appGuestUID": "",
                    "appAccess": True,
                    "doorlockAccess": True,
                    "grmsAccess": False,
                    "guestType": "Co-traveller"
                }
            ]
        )
        response = checkin(orgId, data)
        assert response.status
        assert response.message == 'Complete check-in done successfully'
        assert response.data['check_in_guest_count'] == len(data.guest_list)

    def test_post_check_in_rms(self):
        orgId = '11281'
        db = Firestore('alvie')
        data = CheckinReservationModel(
            reservationId='169410',
            hotelId='11282',
            arrivalDate="2022-02-02 14:00:00",
            departureDate="2022-02-05 11:00:00",
            roomId="14",
            checkin_type="manual",
            doorlockLinked=True,
            grmsLinked=True,
            appLinked=True,
            guest_list=[
                {
                    "name": "Alex",
                    "guestEmail": "alex@gmail.com",
                    "primary": True,
                    "guestPMSId": "1234",
                    "appGuestUID": "CvbKhCxAgQeZ030isXyOHplZaj93",
                    "appAccess": True,
                    "doorlockAccess": True,
                    "grmsAccess": True,
                    "guestType": "Primary"
                },
                {
                    "name": "Harsh Mathur",
                    "guestEmail": "harsh@upswing.global",
                    "primary": False,
                    "guestPMSId": "1235",
                    "appGuestUID": "",
                    "appAccess": True,
                    "doorlockAccess": True,
                    "grmsAccess": False,
                    "guestType": "Co-traveller"
                }
            ]
        )
        response = post_check_in_reservation(db, orgId, data)
        assert response.status



import datetime
import logging
import smtplib
from email.message import EmailMessage
from typing import List, Optional

from googleapiclient.discovery import build
from pydantic import BaseModel

import upswingutil as ul
from upswingutil.db import Mongodb, Firestore
from upswingutil.resource import decrypt
from upswingutil.schema import ResponseDict


class ParameterModel(BaseModel):
    orgId: str
    toList: str
    smtpHost: str
    smtpPort: str
    fromEmail: str  # //use .decode('utf-8')
    password: str  # //use .decode('utf-8')
    subject: str
    template: str
    source: str
    campaignId: str
    reservationId: str


class SendEmailModel(BaseModel):
    orgId: str
    toEmail: str
    smtpHost: str
    smtpPort: str
    fromEmail: str  # //use .decode('utf-8')
    password: str  # //use .decode('utf-8')
    subject: str
    template: str
    source: str
    campaignId: Optional[str]
    reservationId: Optional[str]


class TriggerReservationEmailModel(BaseModel):
    orgId: str
    hotelId: str
    hotelName: str
    reservationId: str
    firstName: str
    lastName: str = ''
    guestEmail: str
    arrivalDate: str
    departureDate: str


def build_template(template: str, v_name: List, r_name: List):
    i = 0
    for v in v_name:
        template = template.replace(v, r_name[i])
        i = i + 1
    return template


def generate_template(template: str, key_value_pair: dict) -> str:
    return template.format(**key_value_pair)


def send_email(record: SendEmailModel):
    response = ResponseDict(status=False, message="Init Message", data={})
    logging.info(f"{record} - {datetime.datetime.utcnow()}")
    mongo = Mongodb(record.orgId)
    try:
        msg = EmailMessage()
        msg['Subject'] = f"{record.subject}"
        msg['From'] = decrypt(record.fromEmail.encode('utf-8'))
        msg['To'] = record.toEmail
        msg.set_content(f"{record.template}", subtype='html')

        with smtplib.SMTP_SSL(record.smtpHost, record.smtpPort) as smtp_client:
            smtp_client.login(decrypt(record.fromEmail.encode('utf-8')),
                              decrypt(record.password.encode('utf-8')))
            smtp_client.send_message(msg)
            smtp_client.quit()
            logging.info(f"Email Sent - {record.toEmail}")
        __post_sending_email__(mongo, record)
        response.status = True
    except Exception as e:
        logging.error(e)
        logging.error(f"Unable to send email to {record.toEmail}")
        _update_report = mongo.get_collection(mongo.OFFERS_CAMPAIGN_COLLECTION).update_one(
            {'_id': record.campaignId}, {
                '$inc': {'reports.email.pushed': 1}}, upsert=True)
        response.status = False
        response.message = e.__str__()
    finally:
        mongo.close_connection()
        return response.dict()


def __post_sending_email__(mongo, record: SendEmailModel):
    try:
        if record.source == 'campaign':
            logging.info('updating campaign report')
            _update_report = mongo.get_collection(mongo.OFFERS_CAMPAIGN_COLLECTION).update_one(
                {'_id': record.campaignId}, {
                    '$inc': {'reports.email.pushed': 1, 'reports.email.delivered': 1}}, upsert=True)

        elif record.source == 'reservation':
            logging.info('updating guest email record')
            __pipeline__ = [
                {
                    '$match': {
                        '_id': record.reservationId
                    }
                }, {
                    '$unwind': {
                        'path': '$guestInfo.guest_list'
                    }
                }, {
                    '$lookup': {
                        'from': 'guests',
                        'localField': 'guestInfo.guest_list.guest',
                        'foreignField': '_id',
                        'as': 'guest_data'
                    }
                }, {
                    '$unwind': {
                        'path': '$guest_data'
                    }
                }, {
                    '$project': {
                        'guestId': '$guestInfo.guest_list.guest',
                        'firstName': '$guest_data.firstName',
                        'lastName': '$guest_data.lastName',
                        'email': '$guest_data.contactInfo.email',
                        'mobile': '$guest_data.contactInfo.mobile'
                    }
                }
            ]
            result = mongo.execute_pipeline(mongo.RESERVATION_COLLECTION, __pipeline__)
            _guest = result[0] if len(result) > 0 else {}
            _update_report = mongo.get_collection(mongo.APP_USERS_MANAGEMENT_COLLECTION).update_one(
                {'firstName': _guest['firstName'], 'lastName': _guest['lastName'], 'email': _guest['email'],
                 'mobile': _guest['mobile'], 'bookings.reservationId': record.reservationId},
                {'$set': {'bookings.$.welcome_email.sendStatus': True,
                          'bookings.$.welcome_email.sentAt': datetime.datetime.utcnow()}}, upsert=True)
    except Exception as e:
        logging.error(e)
        logging.error(f"Unable to perform post email action for {record.toEmail}")


def create_dataflow_job_for_sending_emails(jobname: str, parameters: ParameterModel):
    dataflow = build('dataflow', 'v1b3')
    request = dataflow.projects().locations().templates().launch(
        projectId=ul.G_CLOUD_PROJECT,
        location='asia-south1',
        gcsPath="gs://dataflow-content/Communication/EmailTrigger/templates/email_trigger",
        body={
            'jobName': jobname,
            'parameters': parameters.dict()
        }
    )
    response = request.execute()
    return response


def trigger_reservation_email(data: TriggerReservationEmailModel):
    response = ResponseDict(status=False, message="Init", data={})
    mongo = Mongodb(data.orgId)
    _smtp_record = mongo.get_collection(mongo.INTEGRATION_PROPERTY).find_one(
        {"_id": f"{data.hotelId}-welcome-email", "hotelId": data.hotelId})
    org: dict = Firestore('alvie').get_ref_document(f'Organizations', data.orgId).to_dict()
    if _smtp_record is None:
        logging.error(f"Welcome Email not configured and SMTP Records are none")
    elif _smtp_record['allow_welcome_email']:
        template_data = {
            'logo': org.get('logo'),
            'hotelName': data.hotelName,
            'reservationId': data.reservationId,
            'firstName': data.firstName,
            'lastName': data.lastName,
            'webAppURL': org.get('appURL').get('webAppURL'),
            'googlePlayURL': org.get('appURL').get('googlePlayURL'),
            'appStoreURL': org.get('appURL').get('appStoreURL'),
            'arrivalDate': data.arrivalDate
        }
        _smtp_record['template'] = generate_template(_smtp_record['template'], template_data)
        parameters = SendEmailModel(
            orgId=data.orgId,
            toEmail=data.guestEmail,
            smtpHost=_smtp_record['smtp_host'],
            smtpPort=_smtp_record['smtp_port'],
            fromEmail=f"{_smtp_record['emailId'].decode('utf-8')}",  # to convert it to string
            password=f"{_smtp_record['password'].decode('utf-8')}",  # to convert it to string
            subject=f"Welcome {data.firstName} to {data.hotelName}",
            template=_smtp_record['template'],
            source='reservation',
            campaignId='',
            reservationId=data.reservationId
        )

        _se_flag = send_email(parameters)
        if _se_flag['status']:
            response.status = True
        else:
            response.message = _se_flag['message']

        # res = create_dataflow_job_for_sending_emails(
        #     jobname=f"{data.orgId}-{data.hotelId}-WelcomeEmail-{data.reservationId}-{sent_time}", parameters=parameters)
        # if 'job' in res:
        #     logging.info(f"Successfully Created the Welcome Email Job: {data.guestEmail}")
        #     mongo.close_connection()
        #     return True
        # else:
        #     mongo.close_connection()
        #     return False
    else:
        logging.info(f"Welcome Email disabled for {data.orgId} : {data.hotelId}")

    return response.dict()



import logging
import requests
from upswingutil.db import Firestore
from upswingutil.pms.rms import get_key
from upswingutil.schema import ResponseDict, Token, CheckinReservationModel


def check_in_reservation(orgId: str, data: CheckinReservationModel) -> ResponseDict:
    result = ResponseDict()
    token: Token = get_key(orgId)
    header = {
        'Content-Type': 'application/json',
        'authtoken': token.key
    }
    _url = f"{token.hostName}/reservations/{data.reservationId}/status"
    response = requests.put(_url, headers=header, data="{'status': 'arrived'}")
    if response.status_code != 200:
        logging.error(f'failed to update status in RMS {response.status_code} : {response.reason} : {response.content}')
        result.message = f'failed to update status in RMS {response.status_code} : {response.reason} : {response.content}'
    else:
        result.status = True
        result.data = response.json()
    return result




from loguru import logger
from upswingutil.db import Firestore, Mongodb
from upswingutil.integrations.models import Providers
from upswingutil.schema import ResponseDict, CheckinReservationModel


def grms_activation(orgId: str, data: CheckinReservationModel, auth_token: str) -> ResponseDict:
    """
        Generate grms link for given reservation with respect to which grms provider is linked with the given room
        @author: harsh
        :param orgId:
        :param data: check-in details
        :param auth_token: for accessing upswing's messerchmitt server
        :return: response dict
    """
    response = ResponseDict()
    mongo = Mongodb(orgId)
    try:
        room_mapping = mongo.get_collection(mongo.IOT_ROOM_MAPPING).find_one(
            {'hotelId': data.hotelId, 'pmsRoomId': data.roomId})
        if room_mapping['grmsProvider'] is not None and room_mapping['grmsProvider'] != '' and room_mapping[
            'grmsId'] != '':
            if room_mapping['grmsProvider'] == Providers.MESSERCHMITT.value:
                response = messerchmitt_grms_activation(room_mapping)
            elif room_mapping['grmsProvider'] == Providers.ASSA_ABLOY.value:
                raise Exception(f"GRMS not implemented yet for {room_mapping['grmsProvider']}")
            else:
                raise Exception(f"GRMS not implemented yet for {room_mapping['grmsProvider']}")

            if response.status:
                response = post_grms_linking(orgId, data, response.data)

        else:
            response.message = f'GRMS not mapped for room {orgId} : {data.hotelId} : {data.roomId}'
            logger.info(response.message)
    except Exception as e:
        response.message = f'Error linking GRMS to reservation {orgId} : {data.reservationId}'
        logger.error(response.message)
        logger.error(e)
    finally:
        mongo.close_connection()
        return response


def post_grms_linking(orgId: str, data: CheckinReservationModel, grms: dict) -> ResponseDict:
    """
        This function takes care of linking grms to respective reservation and notifying the guest for the same
        @author: harsh
        :param orgId: the hotel chain
        :param data: check in details
        :param grms: grms linked to the room
        :return: response dict with details
        """
    response = ResponseDict()
    mongo = Mongodb(orgId)
    fireDB = Firestore('alvie')
    fireDB.get_collection(f'Organizations/{orgId}/reservations').document(data.reservationId).set({'grms': grms},
                                                                                                  merge=True)
    result = mongo.get_collection(mongo.RESERVATION_COLLECTION).update_one({'_id': data.reservationId}, {
        '$set': {
            'grms': grms
        }
    }, upsert=True)
    if result.acknowledged:
        response.status = True
        response.message = f'GRMS linked to reservation {orgId} : {data.reservationId}'
        logger.info(response.message)
    return response


def messerchmitt_grms_activation(room_mapping: dict) -> ResponseDict:
    """
        This function gets messerchmitt grms device for the given room guest is staying in
        @author: harsh
        :param room_mapping: doorlock room to which this reservation is linked
        :return: response dict with status
    """
    response = ResponseDict()
    response.status = True
    response.data = {
        'roomId': room_mapping.get('grmsId'),
        'enabled': True
    }
    return response



import unittest
# from upswingutil.ml import ReservationCountForecast
import upswingutil as ul
# from upswingutil.pms.oracle import NAME as ORACLE
# from upswingutil.pms.rms import NAME as RMS
from upswingutil.db import Mongodb

ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
ul.MONGO_URI = "mongodb://AdminUpSWingGlobal:AdMinUpswingtr012325r@dev.db.upswing.global:27017/?authSource=admin&readPreference=primary&appname=MongoDB%20Compass&directConnection=true&ssl=false"
ul.G_CLOUD_PROJECT = "aura-staging-31cae"
ul.FIREBASE = "/Users/harsh/upswing/github/api-oracle/SECRET/aura-staging-31cae-firebase-adminsdk-dyolr-7c135838e9.json"
ul.LOG_LEVEL_VALUE = 'DEBUG'


class TestMongodb(unittest.TestCase):

    def test_connecting_to_mongodb(self):
        mongo = Mongodb('OHIPSB')
        result = mongo.get_collection(mongo.RESERVATION_COLLECTION).count()
        print(f'This collection has {result} records')
        mongo.close_connection()

    def test_query_mongodb(self):
        mongo = Mongodb('11249')
        result = mongo.get_collection(mongo.RESERVATION_COLLECTION).find({'_id': 6},
                                                                         {'accountId': 1, 'activityId': 1, 'areaId': 1,
                                                                          'booking_type': 1})
        for item in result:
            print(item)
        mongo.close_connection()

    def test_aggregate_pipeline(self):
        mongo = Mongodb('11249')
        propertyId = 11264
        __pipeline__ = [
            {
                '$match': {
                    'clientId': propertyId
                }
            }, {
                '$group': {
                    '_id': '$booking_type',
                    'count': {
                        '$sum': 1
                    }
                }
            }
        ]
        result = mongo.execute_pipeline(mongo.RESERVATION_COLLECTION, __pipeline__)
        for item in result:
            print(f'{item["_id"]} has {item["count"]} entries')
        mongo.close_connection()

    ## Before running this uncomment def return_db_collection(self):
    def test_clone_db(self):
        mongo1 = Mongodb('11281')
        mongo2 = Mongodb('tajBusinessBay')

        for coll in mongo1.return_db_collection():
            print(f"{coll} copying")

            result = mongo1.get_collection(f'{coll}').find()
            mongo2.get_collection(f'{coll}').insert_many(result)

            for data in result:
                mongo2.get_collection(mongo2.RESERVATION_COLLECTION).insert_one(data)
            print(f"{coll} copying completed")


if __name__ == '__main__':
    print('testing resv sync')
    TestMongodb()
    print('completed')



import logging

from firebase_admin import auth, get_app
from firebase_admin.auth import UserNotFoundError, EmailAlreadyExistsError


class FirebaseHelper:

    def __init__(self, appName=None):
        self.appName = appName
        self.app = get_app(self.appName)

    def find_user_by_email(self, email):
        try:
            return auth.get_user_by_email(email, app=self.app)
        except UserNotFoundError as userNotFound:
            logging.error(f'User not found in {self.appName} for given email id')
        except Exception as e:
            logging.error(e)

    def create_user_by_email(self, email, name=None):
        user = None
        try:
            user = auth.create_user(email=email, display_name=name, app=self.app)
            logging.info('creating new user in alvie')
        except EmailAlreadyExistsError as err:
            logging.info('returning existing user in alvie')
            user = self.find_user_by_email(email)
        except Exception as e:
            logging.error(e)
        finally:
            return user



import logging
from enum import Enum
from .alvie import store_reservation_to_alvie, store_reservation_to_alvie_v2
from .rms import check_in_reservation as rms_check_in_reservation
from .oracle import check_in_reservation as oracle_check_in_reservation
from .oracle_opera import check_in_reservation as opera_check_in_reservation
from ..db import Firestore, Mongodb
from ..db.model import Status
from ..resource import FirebaseHelper
from ..schema import ResponseDict, CheckinReservationModel


class PMS(str, Enum):
    ORACLE = 'ORACLE'
    RMS = 'RMS'
    OPERA = 'ORACLE-OPERA'


def checkin(orgId: str, data: CheckinReservationModel) -> ResponseDict:
    response = ResponseDict()
    try:
        db = Firestore('alvie')
        pms = db.get_ref_document('Organizations', orgId).to_dict()
        pms = pms['pms']['name']

        if pms == PMS.RMS:
            response = rms_check_in_reservation(orgId, data)
        elif pms == PMS.ORACLE:
            response = oracle_check_in_reservation(db, orgId, data)
        elif pms == PMS.OPERA:
            response = opera_check_in_reservation(db, pms, orgId, data)

        # post successful check-in
        if response.status:
            response = post_check_in_reservation(db, orgId, data)
    except Exception as e:
        logging.error(f'Error check-in pms for {orgId}')
        logging.error(e)
        response.status = False
        response.message = f'Error check-in pms for {orgId}'
    finally:
        return response


def post_check_in_reservation(db: Firestore, orgId: str, data: CheckinReservationModel) -> ResponseDict:
    response = ResponseDict()
    try:
        fb = FirebaseHelper('alvie')
        guest_app_data = {}
        mongo = Mongodb(orgId)
        for guest in data.guest_list:
            uid = guest.appGuestUID
            if uid == '' or uid is None:
                _user = fb.create_user_by_email(guest.guestEmail, guest.name)
                if _user:
                    uid = _user.uid

            guest_app_data[uid] = {
                'name': guest.name,
                'guestEmail': guest.guestEmail,
                'appAccess': guest.appAccess,
                'primary': guest.primary,
                'doorlockAccess': guest.doorlockAccess,
                'grmsAccess': guest.grmsAccess,
                'guestType': guest.guestType,
            }
            logging.debug(f'User {uid} is ready to be saved to reservation {orgId} : {data.reservationId}')
        data_to_update = {
            'status': Status.ARRIVED.value,
            'guest_app_data': guest_app_data
        }
        mongo.get_collection(mongo.RESERVATION_COLLECTION).update_one(
            {'_id': data.reservationId},
            {
                '$set': data_to_update
            },
            upsert=True
        )

        db.get_collection(f'Organizations/{orgId}/reservations').document(data.reservationId).set(data_to_update, merge=True)
        response.status = True
        response.message = f'Complete check-in done successfully for {orgId} : {data.reservationId}'
        response.data = {
            'check_in_guest_count': len(data.guest_list)
        }
        logging.info(response.message)
    except Exception as e:
        response.message = 'Failed in post pms check-in procedure'
        raise e
    finally:
        return response



from enum import Enum
from .email import create_dataflow_job_for_sending_emails, trigger_reservation_email, build_template, \
    TriggerReservationEmailModel, send_email, SendEmailModel

from .notification import NotificationAlvieModel, NotificationAuraModel, push_notification_to_alvie, push_notification_to_aura, NotificationTypes


class CHANNEL(str, Enum):
    EMAIL = 'email'
    WHATSAPP = 'whatsapp'



import json
import logging
import unittest

import firebase_admin
from firebase_admin import credentials

import upswingutil as ul
from upswingutil.resource import access_secret_version, FirebaseHelper, setup_logging


class TestFirestore(unittest.TestCase):
    ul.G_CLOUD_PROJECT = 'aura-staging-31cae'
    ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
    ul.LOG_LEVEL_VALUE = logging.DEBUG

    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Aura', '1'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred)

    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Alvie', '3'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred, name='alvie')

    setup_logging()

    def test_find_user_by_email(self):
        fb = FirebaseHelper('alvie')
        result = fb.find_user_by_email('harsh@upswing.global2')
        if result:
            logging.info(f'User found UID: {result.uid}')

    def test_creating_user_by_email(self):
        fb = FirebaseHelper('alvie')
        result = fb.create_user_by_email('vikash@upswing.global')
        assert result.uid is not None
        if result:
            logging.info(f'User: {result.uid}')



import json
import logging
import unittest

import firebase_admin
from firebase_admin import credentials

import upswingutil as ul
from upswingutil.channel import trigger_reservation_email, TriggerReservationEmailModel, SendEmailModel
from upswingutil.db import Firestore
from upswingutil.resource import access_secret_version


class TestFirestore(unittest.TestCase):
    ul.G_CLOUD_PROJECT = 'aura-staging-31cae'
    ul.ENCRYPTION_SECRET = access_secret_version(ul.G_CLOUD_PROJECT, 'ENCRYPT_SECRET_KEY', 1)
    ul.MONGO_URI = access_secret_version(ul.G_CLOUD_PROJECT, 'MONGOURI', '5')
    ul.LOG_LEVEL_VALUE = logging.DEBUG

    print('Initializing default firebase app')
    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Aura', '1'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred)

    print('Initializing alvie firebase app')
    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Alvie', '3'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred, name='alvie')

    def test_sending_invite(self):
        data = TriggerReservationEmailModel(
            orgId='11281',
            hotelId='11282',
            hotelName='Test 1',
            reservationId='206238',
            firstName='Harsh',
            lastName='Mathur',
            guestEmail='harsh@upswing.global',
            arrivalDate='2022-07-15 14:00:00',
            departureDate='2022-07-30 14:00:00'
        )
        trigger_reservation_email(data)




import json
import logging
import unittest

import firebase_admin
from firebase_admin import credentials

import upswingutil as ul
from upswingutil.db import Firestore
from upswingutil.resource import access_secret_version


class TestFirestore(unittest.TestCase):
    ul.G_CLOUD_PROJECT = 'aura-staging-31cae'
    ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
    ul.LOG_LEVEL_VALUE = logging.DEBUG

    cred = json.loads(
        access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Alvie', '3'))
    cred = credentials.Certificate(cred)
    firebase_admin.initialize_app(cred)

    def test_duplicating_document(self):
        from_path = 'Organizations/11281/properties'
        to_path = 'Organizations/towerPlaza/properties'
        docName = '11282'
        db = Firestore()
        db.duplicate_document(from_path, to_path, docName)

    def test_duplicating_collection(self):
        from_path = 'Organizations/11281/properties/11282/food-and-beverages'
        to_path = 'Organizations/towerPlaza/properties/11282/food-and-beverages'
        db = Firestore()
        db.duplicate_collection(from_path, to_path)
